'''
Created on 17 Nov 2014

@author: swest
'''

class KeyWordSearch():
    def __init__(self):        
        self.master_list = []
        return
    
    def exact(self, pmid, abstract, keywords):
        sentences = abstract.split('. ')
        for sentence in sentences:
            for keyword in keywords:
                if keyword in (sentence.lower()).split():
                    self.master_list.append((pmid, keyword, sentence+'.'))
        return 
    
    def export(self, outputfile):
        outfile = open(outputfile, 'a')
        for line in self.master_list:
            out = '\t'.join(line) + '\n'
            outfile.write(out)
            print(out)
        outfile.close()
        return


'''
Created on 16 Nov 2014

@author: swest
'''

import xml.etree.ElementTree as et
import urllib.request as ur


class NXMLFile():
    def __init__(self, inputfile, html, punctuation, stopwords):
        self.inputfilepath = inputfile
        self.html = html
        self.rem = html + punctuation + stopwords
        return
    
    def parsefile(self):
        if 'http:' in self.inputfilepath or 'ftp' in self.inputfilepath:
            infile = ur.urlopen(self.inputfilepath)
        else:
            infile = open(self.inputfilepath, 'r')
        
        context = et.iterparse(infile, events=('start', 'end'))
        #context = iter(context)
        #print(type(context))
        #event, root = context.next()
        
        pmid = ''
        abstract = ''
        hits = []
        for event, element in context:
            if event == 'end' and \
               (element.tag == 'article-id' or element.tag == 'pub-id') and \
               element.attrib['pub-id-type'] == 'pmid':
                pmid = element.text
            elif event == 'end' and \
               element.tag == 'pub-id':
                pmid = element.text
            #elif event == 'end' and element.tag == 'article-title':
            #    title = element.text
            elif event == 'end' and element.tag == 'abstract':
                abstract = '\n'.join([x.text for x in element.findall('p')])
                element.clear()
                if pmid and abstract:
                    abstract = self.cleaner(abstract)
                    hits.append((pmid, abstract))
                pmid = ''
                abstract = ''
        infile.close()
        return hits
    
    def cleaner(self, abstract):
        #title = ' '.join([x for x in title.split(' ') if x not in self.html])
        abstract = ' '.join([x for x in abstract.split(' ') if x not in self.html])
        return abstract
    
    
        


#!/usr/bin/env python

'''
Author: Oliver Bonham-Carter
Date: 12 November 2014: lister3_ii.py (NXML reader)
Description:
    ***HAS BEEN HEAVILTY MODIFIED BY SEAN --- THE REMAINDER OF THE DESCRIPTION MAY NOT APPLY


    Sean's Copy of Lister
    Program to scan nxml pubmed article summaries files for key words. 
    
    The corpus documents are in nxml format and come from:
    ftp://ftp.ncbi.nlm.nih.gov/pub/pmc/

'''

import sys
from lister import get
from lister import nxml
from lister import keyWordSearch as kws

class Lister():
    def __init__(self):
        return
    
    def main(self):
        g = get.Get(sys.argv)
        if g.args['h'] == 'yes': self.help(); return
        punctuation, html, stopwords, inputfiles, keywords = g.inputs()
        outputfile = g.outputfile()
        
        for inputfile in inputfiles:
            n = nxml.NXMLFile(inputfile, html, punctuation, stopwords)
            hits = n.parsefile()
            k = kws.KeyWordSearch()
            for pmid, abstract in hits:
                k.exact(pmid, abstract, keywords)
            k.export(outputfile)
        return
    
    def help(self):
        p = ''
        p += '\nHELP\n'
        p += '\tARGUMENTS\n'
        p += '\n\t\t-h\t--help\thelp\n'
        p += '\t\t-p argument\t--punctuation=\targument'
        p += '\n\nno i didnt finish writing help'
        print(p)
        return
    
def main():        
    stick = Lister()
    stick.main()        


'''
Created on 16 Nov 2014

@author: swest
'''

import getopt
import os


class Get():
    def __init__(self, args):
        self.args = self.opts(args[1:])
        return
    
    def inputs(self):
        punctuation = self.inputlist(self.args['p'])
        html = self.inputlist(self.args['t'])
        stopwords = self.inputlist(self.args['s'])
        inputfiles = self.inputfiles(self.args['i'])
        keywords = self.inputlist(self.args['k'])
        return punctuation, html, stopwords, inputfiles, keywords
    
    def outputfile(self):
        return self.args['o']
    
    def opts(self, options):
        shortops = 'hp:t:s:i:o:k:'
        longops = ['help', 'punctuation=', 'html=', 'stopwords=', 'input=', 'output=', 'keywords=']
        opts = getopt.getopt(options, shortops, longops)
        
        args = {'h': '', 'p': '', 't': '', 's': '', 'i': '', 'o': '', 'k': ''}
        for (opt, arg) in opts[0]:
            if opt == '-h' or opt == '--help':
                args['h'] = 'yes'
            elif opt == '-p' or opt == '--punctuation':
                args['p'] = arg
            elif opt == '-t' or opt == '--html':
                args['t'] = arg
            elif opt == '-s' or opt == '--stopwords':
                args['s'] = arg
            elif opt == '-i' or opt == '--input':
                args['i'] = arg
            elif opt == '-o' or opt == '--output':
                args['o'] = arg
            elif opt == '-k' or opt == '--keywords':
                args['k'] = arg
        return args

    def inputlist(self, arg):
        inlist = []        
        infile = open(arg, 'r')
        for line in infile:
            inlist.append(line.replace('\n', ''))
        return inlist
    
    def inputfiles(self, arg):
        infiles = [] 
        context = os.walk(arg)
        for walk in context:
            for file in walk[2]:
                if file.endswith('nxml'):
                    tfile = os.path.join(walk[0], file)
                    infiles.append(tfile)
        return infiles
    
    